@@ -17,6 +17,7 @@
  */
 
 #include "Group.h"
+#include "config-keepassx.h"
 
 #include "core/Clock.h"
 #include "core/Config.h"
@@ -25,6 +26,10 @@
 #include "core/Metadata.h"
 #include "core/Tools.h"
 
+#ifdef WITH_XC_KEESHARE
+#include "keeshare/KeeShare.h"
+#endif
+
 #include <QtConcurrent>
 
 const int Group::DefaultIconNumber = 48;
@@ -48,6 +53,7 @@ Group::Group()
 
     connect(m_customData, SIGNAL(customDataModified()), this, SIGNAL(groupModified()));
     connect(this, SIGNAL(groupModified()), SLOT(updateTimeinfo()));
+    connect(this, SIGNAL(groupNonDataChange()), SLOT(updateTimeinfo()));
 }
 
 Group::~Group()
@@ -75,17 +81,6 @@ Group::~Group()
     cleanupParent();
 }
 
-Group* Group::createRecycleBin()
-{
-    Group* recycleBin = new Group();
-    recycleBin->setUuid(QUuid::createUuid());
-    recycleBin->setName(tr("Recycle Bin"));
-    recycleBin->setIcon(RecycleBinIconNumber);
-    recycleBin->setSearchingEnabled(Group::Disable);
-    recycleBin->setAutoTypeEnabled(Group::Disable);
-    return recycleBin;
-}
-
 template <class P, class V> inline bool Group::set(P& property, const V& value)
 {
     if (property != value) {
@@ -138,10 +133,9 @@ QString Group::notes() const
 QImage Group::icon() const
 {
     if (m_data.customIcon.isNull()) {
-        return databaseIcons()->icon(m_data.iconNumber);
+        return databaseIcons()->icon(m_data.iconNumber).toImage();
     } else {
         Q_ASSERT(m_db);
-
         if (m_db) {
             return m_db->metadata()->customIcon(m_data.customIcon);
         } else {
@@ -150,35 +144,28 @@ QImage Group::icon() const
     }
 }
 
-QPixmap Group::iconPixmap() const
+QPixmap Group::iconPixmap(IconSize size) const
 {
+    QPixmap icon(size, size);
     if (m_data.customIcon.isNull()) {
-        return databaseIcons()->iconPixmap(m_data.iconNumber);
+        icon = databaseIcons()->icon(m_data.iconNumber, size);
     } else {
         Q_ASSERT(m_db);
-
         if (m_db) {
-            return m_db->metadata()->customIconPixmap(m_data.customIcon);
-        } else {
-            return QPixmap();
+            icon = m_db->metadata()->customIconPixmap(m_data.customIcon, size);
         }
     }
-}
 
-QPixmap Group::iconScaledPixmap() const
-{
-    if (m_data.customIcon.isNull()) {
-        // built-in icons are 16x16 so don't need to be scaled
-        return databaseIcons()->iconPixmap(m_data.iconNumber);
-    } else {
-        Q_ASSERT(m_db);
-
-        if (m_db) {
-            return m_db->metadata()->customIconScaledPixmap(m_data.customIcon);
-        } else {
-            return QPixmap();
-        }
+    if (isExpired()) {
+        icon = databaseIcons()->applyBadge(icon, DatabaseIcons::Badges::Expired);
     }
+#ifdef WITH_XC_KEESHARE
+    else if (KeeShare::isShared(this)) {
+        icon = KeeShare::indicatorBadge(this, icon);
+    }
+#endif
+
+    return icon;
 }
 
 int Group::iconNumber() const
@@ -257,9 +244,9 @@ Entry* Group::lastTopVisibleEntry() const
     return m_lastTopVisibleEntry;
 }
 
-bool Group::isRecycled()
+bool Group::isRecycled() const
 {
-    Group* group = this;
+    auto group = this;
     if (!group->database()) {
         return false;
     }
@@ -281,6 +268,11 @@ bool Group::isExpired() const
     return m_data.timeInfo.expires() && m_data.timeInfo.expiryTime() < Clock::currentDateTimeUtc();
 }
 
+bool Group::isEmpty() const
+{
+    return !hasChildren() && m_entries.isEmpty();
+}
+
 CustomData* Group::customData()
 {
     return m_customData;
@@ -370,11 +362,7 @@ void Group::setExpanded(bool expanded)
 {
     if (m_data.isExpanded != expanded) {
         m_data.isExpanded = expanded;
-        if (config()->get("IgnoreGroupExpansion").toBool()) {
-            updateTimeinfo();
-            return;
-        }
-        emit groupModified();
+        emit groupNonDataChange();
     }
 }
 
@@ -457,8 +445,8 @@ void Group::setParent(Group* parent, int index)
             recCreateDelObjects();
 
             // copy custom icon to the new database
-            if (!iconUuid().isNull() && parent->m_db && m_db->metadata()->containsCustomIcon(iconUuid())
-                && !parent->m_db->metadata()->containsCustomIcon(iconUuid())) {
+            if (!iconUuid().isNull() && parent->m_db && m_db->metadata()->hasCustomIcon(iconUuid())
+                && !parent->m_db->metadata()->hasCustomIcon(iconUuid())) {
                 parent->m_db->metadata()->addCustomIcon(iconUuid(), icon());
             }
         }
@@ -504,16 +492,25 @@ void Group::setParent(Database* db)
     QObject::setParent(db);
 }
 
-QStringList Group::hierarchy() const
+QStringList Group::hierarchy(int height) const
 {
     QStringList hierarchy;
     const Group* group = this;
     const Group* parent = m_parent;
+
+    if (height == 0) {
+        return hierarchy;
+    }
+
     hierarchy.prepend(group->name());
 
-    while (parent) {
+    int level = 1;
+    bool heightReached = level == height;
+
+    while (parent && !heightReached) {
         group = group->parentGroup();
         parent = group->parentGroup();
+        heightReached = ++level == height;
 
         hierarchy.prepend(group->name());
     }
@@ -521,6 +518,11 @@ QStringList Group::hierarchy() const
     return hierarchy;
 }
 
+bool Group::hasChildren() const
+{
+    return !children().isEmpty();
+}
+
 Database* Group::database()
 {
     return m_db;
@@ -577,13 +579,18 @@ QList<Entry*> Group::referencesRecursive(const Entry* entry) const
                                           [entry](const Entry* e) { return e->hasReferencesTo(entry->uuid()); });
 }
 
-Entry* Group::findEntryByUuid(const QUuid& uuid) const
+Entry* Group::findEntryByUuid(const QUuid& uuid, bool recursive) const
 {
     if (uuid.isNull()) {
         return nullptr;
     }
 
-    for (Entry* entry : entriesRecursive(false)) {
+    auto entries = m_entries;
+    if (recursive) {
+        entries = entriesRecursive(false);
+    }
+
+    for (auto entry : entries) {
         if (entry->uuid() == uuid) {
             return entry;
         }
@@ -715,25 +722,34 @@ Group* Group::findGroupByPathRecursive(const QString& groupPath, const QString&
     return nullptr;
 }
 
-QString Group::print(bool recursive, int depth)
+QString Group::print(bool recursive, bool flatten, int depth)
 {
-
     QString response;
-    QString indentation = QString("  ").repeated(depth);
+    QString prefix;
+
+    if (flatten) {
+        const QString separator("/");
+        prefix = hierarchy(depth).join(separator);
+        if (!prefix.isEmpty()) {
+            prefix += separator;
+        }
+    } else {
+        prefix = QString("  ").repeated(depth);
+    }
 
     if (entries().isEmpty() && children().isEmpty()) {
-        response += indentation + tr("[empty]", "group has no children") + "\n";
+        response += prefix + tr("[empty]", "group has no children") + "\n";
         return response;
     }
 
     for (Entry* entry : entries()) {
-        response += indentation + entry->title() + "\n";
+        response += prefix + entry->title() + "\n";
     }
 
     for (Group* innerGroup : children()) {
-        response += indentation + innerGroup->name() + "/\n";
+        response += prefix + innerGroup->name() + "/\n";
         if (recursive) {
-            response += innerGroup->print(recursive, depth + 1);
+            response += innerGroup->print(recursive, flatten, depth + 1);
         }
     }
 
@@ -790,6 +806,42 @@ QSet<QUuid> Group::customIconsRecursive() const
     return result;
 }
 
+QList<QString> Group::usernamesRecursive(int topN) const
+{
+    // Collect all usernames and sort for easy counting
+    QHash<QString, int> countedUsernames;
+    for (const auto* entry : entriesRecursive()) {
+        const auto username = entry->username();
+        if (!username.isEmpty() && !entry->isAttributeReference(EntryAttributes::UserNameKey)) {
+            countedUsernames.insert(username, ++countedUsernames[username]);
+        }
+    }
+
+    // Sort username/frequency pairs by frequency and name
+    QList<QPair<QString, int>> sortedUsernames;
+    for (const auto& key : countedUsernames.keys()) {
+        sortedUsernames.append({key, countedUsernames[key]});
+    }
+
+    auto comparator = [](const QPair<QString, int>& arg1, const QPair<QString, int>& arg2) {
+        if (arg1.second == arg2.second) {
+            return arg1.first < arg2.first;
+        }
+        return arg1.second > arg2.second;
+    };
+
+    std::sort(sortedUsernames.begin(), sortedUsernames.end(), comparator);
+
+    // Take first topN usernames if set
+    QList<QString> usernames;
+    int actualUsernames = topN < 0 ? sortedUsernames.size() : std::min(topN, sortedUsernames.size());
+    for (int i = 0; i < actualUsernames; i++) {
+        usernames.append(sortedUsernames[i].first);
+    }
+
+    return usernames;
+}
+
 Group* Group::findGroupByUuid(const QUuid& uuid)
 {
     if (uuid.isNull()) {
@@ -906,17 +958,36 @@ void Group::removeEntry(Entry* entry)
     emit entryRemoved(entry);
 }
 
+void Group::moveEntryUp(Entry* entry)
+{
+    int row = m_entries.indexOf(entry);
+    if (row <= 0) {
+        return;
+    }
+
+    emit entryAboutToMoveUp(row);
+    m_entries.move(row, row - 1);
+    emit entryMovedUp();
+    emit groupNonDataChange();
+}
+
+void Group::moveEntryDown(Entry* entry)
+{
+    int row = m_entries.indexOf(entry);
+    if (row >= m_entries.size() - 1) {
+        return;
+    }
+
+    emit entryAboutToMoveDown(row);
+    m_entries.move(row, row + 1);
+    emit entryMovedDown();
+    emit groupNonDataChange();
+}
+
 void Group::connectDatabaseSignalsRecursive(Database* db)
 {
     if (m_db) {
-        disconnect(SIGNAL(groupDataChanged(Group*)), m_db);
-        disconnect(SIGNAL(groupAboutToRemove(Group*)), m_db);
-        disconnect(SIGNAL(groupRemoved()), m_db);
-        disconnect(SIGNAL(groupAboutToAdd(Group*, int)), m_db);
-        disconnect(SIGNAL(groupAdded()), m_db);
-        disconnect(SIGNAL(aboutToMove(Group*, Group*, int)), m_db);
-        disconnect(SIGNAL(groupMoved()), m_db);
-        disconnect(SIGNAL(groupModified()), m_db);
+        disconnect(m_db);
     }
 
     for (Entry* entry : asConst(m_entries)) {
@@ -938,6 +1009,7 @@ void Group::connectDatabaseSignalsRecursive(Database* db)
         connect(this, SIGNAL(aboutToMove(Group*,Group*,int)), db, SIGNAL(groupAboutToMove(Group*,Group*,int)));
         connect(this, SIGNAL(groupMoved()), db, SIGNAL(groupMoved()));
         connect(this, SIGNAL(groupModified()), db, SLOT(markAsModified()));
+        connect(this, SIGNAL(groupNonDataChange()), db, SLOT(markNonDataChange()));
         // clang-format on
     }
 
@@ -1057,6 +1129,74 @@ Entry* Group::addEntryWithPath(const QString& entryPath)
     return entry;
 }
 
+void Group::applyGroupIconOnCreateTo(Entry* entry)
+{
+    Q_ASSERT(entry);
+
+    if (!config()->get(Config::UseGroupIconOnEntryCreation).toBool()) {
+        return;
+    }
+
+    if (iconNumber() == Group::DefaultIconNumber && iconUuid().isNull()) {
+        return;
+    }
+
+    applyGroupIconTo(entry);
+}
+
+void Group::applyGroupIconTo(Entry* entry)
+{
+    Q_ASSERT(entry);
+
+    if (iconUuid().isNull()) {
+        entry->setIcon(iconNumber());
+    } else {
+        entry->setIcon(iconUuid());
+    }
+}
+
+void Group::applyGroupIconTo(Group* other)
+{
+    Q_ASSERT(other);
+
+    if (iconUuid().isNull()) {
+        other->setIcon(iconNumber());
+    } else {
+        other->setIcon(iconUuid());
+    }
+}
+
+void Group::applyGroupIconToChildGroups()
+{
+    for (Group* recursiveChild : groupsRecursive(false)) {
+        applyGroupIconTo(recursiveChild);
+    }
+}
+
+void Group::applyGroupIconToChildEntries()
+{
+    for (Entry* recursiveEntry : entriesRecursive(false)) {
+        applyGroupIconTo(recursiveEntry);
+    }
+}
+
+void Group::sortChildrenRecursively(bool reverse)
+{
+    std::sort(
+        m_children.begin(), m_children.end(), [reverse](const Group* childGroup1, const Group* childGroup2) -> bool {
+            QString name1 = childGroup1->name();
+            QString name2 = childGroup2->name();
+            return reverse ? name1.compare(name2, Qt::CaseInsensitive) > 0
+                           : name1.compare(name2, Qt::CaseInsensitive) < 0;
+        });
+
+    for (auto child : m_children) {
+        child->sortChildrenRecursively(reverse);
+    }
+
+    emit groupModified();
+}
+
 bool Group::GroupData::operator==(const Group::GroupData& other) const
 {
     return equals(other, CompareItemDefault);
