@@ -130,18 +130,20 @@ QVariant GroupModel::data(const QModelIndex& index, int role) const
 #endif
         return nameTemplate.arg(group->name());
     } else if (role == Qt::DecorationRole) {
-        QPixmap pixmap = group->isExpired() ? databaseIcons()->iconPixmap(DatabaseIcons::ExpiredIconIndex)
-                                            : group->iconScaledPixmap();
-#if defined(WITH_XC_KEESHARE)
-        pixmap = KeeShare::indicatorBadge(group, pixmap);
-#endif
-        return pixmap;
+        return group->iconPixmap();
     } else if (role == Qt::FontRole) {
         QFont font;
         if (group->isExpired()) {
             font.setStrikeOut(true);
         }
         return font;
+    } else if (role == Qt::ToolTipRole) {
+        QString tooltip;
+        if (!group->parentGroup()) {
+            // only show a tooltip for the root group
+            tooltip = m_db->filePath();
+        }
+        return tooltip;
     } else {
         return QVariant();
     }
@@ -297,7 +299,7 @@ bool GroupModel::dropMimeData(const QMimeData* data,
             Database* targetDb = parentGroup->database();
             QUuid customIcon = entry->iconUuid();
 
-            if (sourceDb != targetDb && !customIcon.isNull() && !targetDb->metadata()->containsCustomIcon(customIcon)) {
+            if (sourceDb != targetDb && !customIcon.isNull() && !targetDb->metadata()->hasCustomIcon(customIcon)) {
                 targetDb->metadata()->addCustomIcon(customIcon, sourceDb->metadata()->customIcon(customIcon));
             }
 
@@ -410,3 +412,30 @@ void GroupModel::groupMoved()
 {
     endMoveRows();
 }
+
+void GroupModel::sortChildren(Group* rootGroup, bool reverse)
+{
+    emit layoutAboutToBeChanged();
+
+    QList<QModelIndex> oldIndexes;
+    collectIndexesRecursively(oldIndexes, rootGroup->children());
+
+    rootGroup->sortChildrenRecursively(reverse);
+
+    QList<QModelIndex> newIndexes;
+    collectIndexesRecursively(newIndexes, rootGroup->children());
+
+    for (int i = 0; i < oldIndexes.count(); i++) {
+        changePersistentIndex(oldIndexes[i], newIndexes[i]);
+    }
+
+    emit layoutChanged();
+}
+
+void GroupModel::collectIndexesRecursively(QList<QModelIndex>& indexes, QList<Group*> groups)
+{
+    for (auto group : groups) {
+        indexes.append(index(group));
+        collectIndexesRecursively(indexes, group->children());
+    }
+}
