@@ -50,18 +50,21 @@ int main(int argc, char** argv)
 
 #if QT_VERSION >= QT_VERSION_CHECK(5, 6, 0)
     QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+    QGuiApplication::setAttribute(Qt::AA_UseHighDpiPixmaps);
 #endif
-#ifdef Q_OS_LINUX
-    if (qgetenv("XDG_SESSION_TYPE") == QByteArrayLiteral("wayland")) {
-        qWarning() << "Warning: disregarding XDG_SESSION_TYPE=wayland";
-        qWarning() << "To use wayland anyway, please set QT_QPA_PLATFORM=wayland";
-        qunsetenv("XDG_SESSION_TYPE");
-    }
+#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0) && defined(Q_OS_WIN)
+    QGuiApplication::setHighDpiScaleFactorRoundingPolicy(Qt::HighDpiScaleFactorRoundingPolicy::PassThrough);
 #endif
 
     Application app(argc, argv);
-    Application::setApplicationName("keepassxc");
+    Application::setApplicationName("KeePassXC");
     Application::setApplicationVersion(KEEPASSXC_VERSION);
+    app.setProperty("KPXC_QUALIFIED_APPNAME", "org.keepassxc.KeePassXC");
+    app.applyTheme();
+#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+    QGuiApplication::setDesktopFileName(app.property("KPXC_QUALIFIED_APPNAME").toString() + QStringLiteral(".desktop"));
+#endif
+
     // don't set organizationName as that changes the return value of
     // QStandardPaths::writableLocation(QDesktopServices::DataLocation)
     Bootstrap::bootstrapApplication();
@@ -82,8 +85,7 @@ int main(int argc, char** argv)
 
     QCommandLineOption helpOption = parser.addHelpOption();
     QCommandLineOption versionOption = parser.addVersionOption();
-    QCommandLineOption debugInfoOption(QStringList() << "debug-info",
-                                       QObject::tr("Displays debugging information."));
+    QCommandLineOption debugInfoOption(QStringList() << "debug-info", QObject::tr("Displays debugging information."));
     parser.addOption(configOption);
     parser.addOption(keyfileOption);
     parser.addOption(pwstdinOption);
@@ -144,9 +146,9 @@ int main(int argc, char** argv)
         if (pwstdin) {
             // we always need consume a line of STDIN if --pw-stdin is set to clear out the
             // buffer for native messaging, even if the specified file does not exist
-            static QTextStream in(stdin, QIODevice::ReadOnly);
-            static QTextStream out(stdout, QIODevice::WriteOnly);
+            QTextStream out(stdout, QIODevice::WriteOnly);
             out << QObject::tr("Database password: ") << flush;
+            Utils::setDefaultTextStreams();
             password = Utils::getPassword();
         }
 
@@ -157,6 +159,11 @@ int main(int argc, char** argv)
 
     int exitCode = Application::exec();
 
+    // Check if restart was requested
+    if (exitCode == RESTART_EXITCODE) {
+        QProcess::startDetached(QCoreApplication::applicationFilePath(), {});
+    }
+
 #if defined(WITH_ASAN) && defined(WITH_LSAN)
     // do leak check here to prevent massive tail of end-of-process leak errors from third-party libraries
     __lsan_do_leak_check();
